@@ -1,6 +1,5 @@
 # Copyright 1999-2005 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/init.d/halt.sh,v 1.64.2.7 2005/05/15 22:46:03 vapier Exp $
 
 # Check to see if this is a livecd, if it is read the commandline
 # this mainly makes sure $CDBOOT is defined if it's a livecd
@@ -14,63 +13,8 @@ if [[ -x /sbin/pam_console_apply && ! -c /dev/.devfsd && \
 	/sbin/pam_console_apply -r
 fi
 
-# We need to properly terminate devfsd to save the permissions
-if [[ -n $(ps --no-heading -C 'devfsd') ]]; then
-	ebegin "Stopping devfsd"
-	killall -15 devfsd &>/dev/null
-	eend $?
-elif [[ ! -e /dev/.devfsd && -e /dev/.udev && -z ${CDBOOT} && \
-        ${RC_DEVICE_TARBALL} == "yes" ]] && \
-		touch /lib/udev-state/devices.tar.bz2 2>/dev/null
-then
-	ebegin "Saving device nodes"
-	# Handle our temp files
-	devices_udev=$(mktemp /tmp/devices.udev.XXXXXX)
-	devices_real=$(mktemp /tmp/devices.real.XXXXXX)
-	device_tarball=$(mktemp /tmp/devices-XXXXXX)
-	
-	if [[ -z ${devices_udev} || -z ${devices_real} || \
-	      -z ${device_tarball} ]]; then
-		eend 1 "Could not create temporary files!"
-	else
-		cd /dev
-		# Find all devices
-		find . -xdev -type b -or -type c -or -type l | cut -d/ -f2- > \
-			"${devices_real}"
-		# Figure out what udev created
-		eval $(grep '^[[:space:]]*udev_db=' /etc/udev/udev.conf)
-		if [[ -d ${udev_db} ]]; then
-			# New udev_db is clear text ...
-			udevinfo=$(cat "${udev_db}"/*)
-		else
-			# Old one is not ...
-			udevinfo=$(udevinfo -d)
-		fi
-		# This basically strips 'S:' and 'N:' from the db output, and then
-		# print all the nodes/symlinks udev created ...
-		echo "${udevinfo}" | gawk '
-			/^(N|S):.+/ {
-				sub(/^(N|S):/, "")
-				split($0, nodes)
-				for (x in nodes)
-					print nodes[x]
-			}' > "${devices_udev}"
-		# These ones we also do not want in there
-		for x in MAKEDEV core fd initctl pts shm stderr stdin stdout; do
-			echo "${x}" >> "${devices_udev}"
-		done
-		tarball_devices="$(fgrep -x -v -f "${devices_udev}" < "${devices_real}")"
-		# Now only tarball those not created by udev if we have any
-		if [[ -n ${tarball_devices} ]]; then
-			try tar -jclpf "${device_tarball}" ${tarball_devices}
-			try mv -f "${device_tarball}" /lib/udev-state/devices.tar.bz2
-			try rm -f "${devices_udev}" "${devices_real}"
-		else
-			rm -f /lib/udev-state/devices.tar.bz2
-		fi
-		eend 0
-	fi
-fi
+stop_addon devfs
+stop_addon udev
 
 # Try to unmount all tmpfs filesystems not in use, else a deadlock may
 # occure, bug #13599.
@@ -193,10 +137,10 @@ mount_readonly() {
 	sync; sync
 	sleep 1
 
-	for x in $(awk '$1 != "none" { print $2 }' /proc/mounts | sort -r) ; do
+	for x in $(awk '$1 != "none" { print $2 }' /proc/mounts | sort -ur) ; do
 		x=${x//\\040/ }
 		if [[ ${cmd} == "u" ]]; then
-			umount -r -r "${x}"
+			umount -n -r "${x}"
 		else
 			mount -n -o remount,ro "${x}" &>/dev/null
 		fi
