@@ -1,18 +1,20 @@
 # Copyright 1999-2005 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.81.2.6 2005/05/15 20:00:31 vapier Exp $
 
 RC_GOT_FUNCTIONS="yes"
 
+# Override defaults with user settings ...
+[ -f /etc/conf.d/rc ] && source /etc/conf.d/rc
+
 # daemontools dir
 SVCDIR="/var/lib/supervise"
 
 # Check /etc/conf.d/rc for a description of these ...
-svcdir="/var/lib/init.d"
 svclib="/lib/rcscripts"
-svcmount="no"
-svcfstype="tmpfs"
-svcsize=1024
+svcdir=${svcdir:-/var/lib/init.d}
+svcmount=${svcmount:-no}
+svcfstype=${svcfstype:-tmpfs}
+svcsize=${svcsize:-1024}
 
 # Different types of dependencies
 deptypes="need use"
@@ -25,20 +27,24 @@ ordtypes="before after"
 
 # Dont output to stdout?
 RC_QUIET_STDOUT="no"
-RC_VERBOSE="${RC_VERBOSE:-no}"
+RC_VERBOSE=${RC_VERBOSE:-no}
 
 # Should we use color?
-RC_NOCOLOR="${RC_NOCOLOR:-no}"
+RC_NOCOLOR=${RC_NOCOLOR:-no}
 # Can the terminal handle endcols?
 RC_ENDCOL="yes"
 
 #
 # Default values for rc system
 #
-RC_TTY_NUMBER=11
-RC_NET_STRICT_CHECKING="no"
-RC_PARALLEL_STARTUP="no"
-RC_USE_CONFIG_PROFILE="yes"
+RC_TTY_NUMBER=${RC_TTY_NUMBER:-11}
+RC_PARALLEL_STARTUP=${RC_PARALLEL_STARTUP:-no}
+RC_NET_STRICT_CHECKING=${RC_NET_STRICT_CHECKING:-no}
+RC_VOLUME_ORDER=${RC_VOLUME_ORDER:-raid evms lvm dm}
+RC_USE_FSTAB=${RC_USE_FSTAB:-no}
+RC_USE_CONFIG_PROFILE=${RC_USE_CONFIG_PROFILE:-yes}
+RC_FORCE_AUTO=${RC_FORCE_AUTO:-no}
+RC_DEVICES=${RC_DEVICES:-auto}
 
 #
 # Default values for e-message indentation and dots
@@ -48,9 +54,6 @@ RC_DEFAULT_INDENT=2
 #RC_DOT_PATTERN=' .'
 RC_DOT_PATTERN=''
 
-# Override defaults with user settings ...
-[ -f /etc/conf.d/rc ] && source /etc/conf.d/rc
-
 # void import_addon(char *addon)
 #
 #  Import code from the specified addon if it exists
@@ -86,6 +89,14 @@ profiling() {
 }
 import_addon profiling-functions.sh
 
+# void bootlog(...)
+#
+#  Notify bootlogger about important events.
+bootlog() {
+	return 0
+}
+[[ ${RC_BOOTLOG} == "yes" ]] && import_addon bootlogger.sh
+
 # void get_bootconfig()
 #
 #    Get the BOOTLEVEL and SOFTLEVEL by setting
@@ -97,25 +108,27 @@ get_bootconfig() {
 	local newbootlevel=
 	local newsoftlevel=
 
-	for copt in $(</proc/cmdline) ; do
-		case "${copt%=*}" in
-			"bootlevel")
-				newbootlevel="${copt##*=}"
-				;;
-			"softlevel")
-				newsoftlevel="${copt##*=}"
-				;;
-		esac
-	done
+	if [[ -r /proc/cmdline ]] ; then
+		for copt in $(</proc/cmdline) ; do
+			case "${copt%=*}" in
+				bootlevel)
+					newbootlevel=${copt##*=}
+					;;
+				softlevel)
+					newsoftlevel=${copt##*=}
+					;;
+			esac
+		done
+	fi
 
-	if [ -n "${newbootlevel}" ] ; then
-		export BOOTLEVEL="${newbootlevel}"
+	if [[ -n ${newbootlevel} ]] ; then
+		export BOOTLEVEL=${newbootlevel}
 	else
 		export BOOTLEVEL="boot"
 	fi
 
-	if [ -n "${newsoftlevel}" ] ; then
-		export DEFAULTLEVEL="${newsoftlevel}"
+	if [[ -n ${newsoftlevel} ]] ; then
+		export DEFAULTLEVEL=${newsoftlevel}
 	else
 		export DEFAULTLEVEL="default"
 	fi
@@ -597,7 +610,7 @@ get_base_ver() {
 # Network filesystems list for common use in rc-scripts.
 # This variable is used in is_net_fs and other places such as
 # localmount.
-NET_FS_LIST="afs cifs coda ncpfs nfs nfs4 shfs smbfs"
+NET_FS_LIST="afs cifs coda davfs gfs ncpfs nfs nfs4 ocfs2 shfs smbfs"
 
 # bool is_net_fs(path)
 #
@@ -606,7 +619,13 @@ NET_FS_LIST="afs cifs coda ncpfs nfs nfs4 shfs smbfs"
 #   EXAMPLE:  if is_net_fs / ; then ...
 #
 is_net_fs() {
-	local fstype=$(mount -o remount -fv "$1" | awk '{print $(NF-1)}')
+	local fstype
+	# /proc/mounts is always accurate but may not always be available
+	if [[ -e /proc/mounts ]]; then
+		fstype=$( sed -n -e '/^rootfs/!s:.* '"$1"' \([^ ]*\).*:\1:p' /proc/mounts )
+	else
+		fstype=$( mount | sed -n -e 's:.* on '"$1"' type \([^ ]*\).*:\1:p' )
+	fi
 	[[ " ${NET_FS_LIST} " == *" ${fstype} "* ]]
 	return $?
 }
@@ -633,6 +652,16 @@ is_vserver_sys() {
 	return $?
 }
 
+# bool is_xenU_sys()
+#
+#   return 0 if the currently running system is an unprivileged Xen domain
+#
+#   EXAMPLE:  if is_xenU_sys ; then ...
+#
+is_xenU_sys() {
+	[[ -d /proc/xen && ! -f /proc/xen/privcmd ]]
+}
+
 # bool get_mount_fstab(path)
 #
 #   return the parameters to pass to the mount command generated from fstab
@@ -798,9 +827,9 @@ else
 	GOOD=$'\e[32;01m'
 	WARN=$'\e[33;01m'
 	BAD=$'\e[31;01m'
-	NORMAL=$'\e[0m'
 	HILITE=$'\e[36;01m'
 	BRACKET=$'\e[34;01m'
+	NORMAL=$'\e[0m'
 fi
 
 ##############################################################################
