@@ -102,10 +102,8 @@ svc_stop() {
 	if ! service_started "${myservice}" ; then
 		if [[ ${RC_QUIET_STDOUT} != "yes" ]] ; then
 			eerror "ERROR:  \"${myservice}\" has not yet been started."
-			return 1
-		else
-			return 0
 		fi
+		return 0
 	fi
 
 	# Do not try to stop if it had already failed to do so on runlevel change
@@ -367,13 +365,13 @@ source "${myscript}"
 opts="start stop restart pause zap ${opts}"
 
 svc_homegrown() {
-	local arg="$1"
-	local x=
+	local x arg=$1
+	shift
 
 	# Walk through the list of available options, looking for the
 	# requested one.
 	for x in ${opts} ; do
-		if [[ ${x} == ${arg} ]] ; then
+		if [[ ${x} == "${arg}" ]] ; then
 			if typeset -F "${x}" &>/dev/null ; then
 				# Run the homegrown function
 				"${x}"
@@ -382,9 +380,11 @@ svc_homegrown() {
 			fi
 		fi
 	done
+	x=""
 
 	# If we're here, then the function wasn't in $opts.
-	eerror "ERROR:  wrong args. (  "${arg}" / $* )"
+	[[ -n $* ]] && x="/ $* "
+	eerror "ERROR: wrong args ( "${arg}" ${x})"
 	# Do not quote this either ...
 	usage ${opts}
 	exit 1
@@ -480,6 +480,9 @@ for arg in $* ; do
 		;;
 	--quiet|--nocolor)
 		;;
+	help)
+		exec "${svclib}"/sh/rc-help.sh "${myscript}" help
+		;;
 	*)
 		# Allow for homegrown functions
 		svc_homegrown ${arg}
